@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2018 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -342,7 +342,8 @@ namespace Orthanc
   }
 
 
-  void ServerIndex::FlushThread(ServerIndex* that)
+  void ServerIndex::FlushThread(ServerIndex* that,
+                                unsigned int threadSleep)
   {
     // By default, wait for 10 seconds before flushing
     unsigned int sleep = 10;
@@ -368,7 +369,7 @@ namespace Orthanc
 
     while (!that->done_)
     {
-      boost::this_thread::sleep(boost::posix_time::seconds(1));
+      boost::this_thread::sleep(boost::posix_time::milliseconds(threadSleep));
       count++;
       if (count < sleep)
       {
@@ -538,7 +539,8 @@ namespace Orthanc
 
 
   ServerIndex::ServerIndex(ServerContext& context,
-                           IDatabaseWrapper& db) : 
+                           IDatabaseWrapper& db,
+                           unsigned int threadSleep) : 
     done_(false),
     db_(db),
     maximumStorageSize_(0),
@@ -555,10 +557,11 @@ namespace Orthanc
 
     if (db.HasFlushToDisk())
     {
-      flushThread_ = boost::thread(FlushThread, this);
+      flushThread_ = boost::thread(FlushThread, this, threadSleep);
     }
 
-    unstableResourcesMonitorThread_ = boost::thread(UnstableResourcesMonitorThread, this);
+    unstableResourcesMonitorThread_ = boost::thread
+      (UnstableResourcesMonitorThread, this, threadSleep);
   }
 
 
@@ -770,7 +773,7 @@ namespace Orthanc
       }
 
       // Attach the auto-computed metadata for the patient/study/series levels
-      std::string now = SystemToolbox::GetNowIsoString();
+      std::string now = SystemToolbox::GetNowIsoString(true /* use UTC time (not local time) */);
       db_.SetMetadata(series, MetadataType_LastUpdate, now);
       db_.SetMetadata(study, MetadataType_LastUpdate, now);
       db_.SetMetadata(patient, MetadataType_LastUpdate, now);
@@ -778,16 +781,37 @@ namespace Orthanc
       // Attach the auto-computed metadata for the instance level,
       // reflecting these additions into the input metadata map
       SetInstanceMetadata(instanceMetadata, instance, MetadataType_Instance_ReceptionDate, now);
-      SetInstanceMetadata(instanceMetadata, instance, MetadataType_Instance_RemoteAet, instanceToStore.GetRemoteAet());
+      SetInstanceMetadata(instanceMetadata, instance, MetadataType_Instance_RemoteAet,
+                          instanceToStore.GetOrigin().GetRemoteAetC());
       SetInstanceMetadata(instanceMetadata, instance, MetadataType_Instance_Origin, 
-                          EnumerationToString(instanceToStore.GetRequestOrigin()));
-        
+                          EnumerationToString(instanceToStore.GetOrigin().GetRequestOrigin()));
+
       {
         std::string s;
+
         if (instanceToStore.LookupTransferSyntax(s))
         {
+          // New in Orthanc 1.2.0
           SetInstanceMetadata(instanceMetadata, instance, MetadataType_Instance_TransferSyntax, s);
         }
+
+        if (instanceToStore.GetOrigin().LookupRemoteIp(s))
+        {
+          // New in Orthanc 1.4.0
+          SetInstanceMetadata(instanceMetadata, instance, MetadataType_Instance_RemoteIp, s);
+        }
+
+        if (instanceToStore.GetOrigin().LookupCalledAet(s))
+        {
+          // New in Orthanc 1.4.0
+          SetInstanceMetadata(instanceMetadata, instance, MetadataType_Instance_CalledAet, s);
+        }
+
+        if (instanceToStore.GetOrigin().LookupHttpUsername(s))
+        {
+          // New in Orthanc 1.4.0
+          SetInstanceMetadata(instanceMetadata, instance, MetadataType_Instance_HttpUsername, s);
+        }
       }
 
       const DicomValue* value;
@@ -1182,7 +1206,12 @@ namespace Orthanc
 
     {
       boost::mutex::scoped_lock lock(mutex_);
+
+      // Fix wrt. Orthanc <= 1.3.2: A transaction was missing, as
+      // "GetLastChange()" involves calls to "GetPublicId()"
+      Transaction transaction(*this);
       db_.GetChanges(changes, done, since, maxResults);
+      transaction.Commit(0);
     }
 
     FormatLog(target, changes, "Changes", done, since);
@@ -1195,7 +1224,12 @@ namespace Orthanc
 
     {
       boost::mutex::scoped_lock lock(mutex_);
+
+      // Fix wrt. Orthanc <= 1.3.2: A transaction was missing, as
+      // "GetLastChange()" involves calls to "GetPublicId()"
+      Transaction transaction(*this);
       db_.GetLastChange(changes);
+      transaction.Commit(0);
     }
 
     FormatLog(target, changes, "Changes", true, 0);
@@ -1212,7 +1246,7 @@ namespace Orthanc
     ResourceType type;
     if (!db_.LookupResource(id, type, publicId))
     {
-      throw OrthancException(ErrorCode_InternalError);
+      throw OrthancException(ErrorCode_InexistentItem);
     }
 
     std::string patientId;
@@ -1281,7 +1315,7 @@ namespace Orthanc
                               type,
                               publicId,
                               remoteModality,
-                              SystemToolbox::GetNowIsoString(),
+                              SystemToolbox::GetNowIsoString(true /* use UTC time (not local time) */),
                               patientId,
                               studyInstanceUid,
                               seriesInstanceUid,
@@ -1877,7 +1911,8 @@ namespace Orthanc
   }
 
 
-  void ServerIndex::UnstableResourcesMonitorThread(ServerIndex* that)
+  void ServerIndex::UnstableResourcesMonitorThread(ServerIndex* that,
+                                                   unsigned int threadSleep)
   {
     int stableAge = Configuration::GetGlobalUnsignedIntegerParameter("StableAge", 60);
     if (stableAge <= 0)
@@ -1889,8 +1924,8 @@ namespace Orthanc
 
     while (!that->done_)
     {
-      // Check for stable resources each second
-      boost::this_thread::sleep(boost::posix_time::seconds(1));
+      // Check for stable resources each few seconds
+      boost::this_thread::sleep(boost::posix_time::milliseconds(threadSleep));
 
       boost::mutex::scoped_lock lock(that->mutex_);
 
@@ -2091,13 +2126,20 @@ namespace Orthanc
   }
 
 
+  bool ServerIndex::LookupGlobalProperty(std::string& value,
+                                         GlobalProperty property)
+  {
+    boost::mutex::scoped_lock lock(mutex_);
+    return db_.LookupGlobalProperty(value, property);
+  }
+  
+
   std::string ServerIndex::GetGlobalProperty(GlobalProperty property,
                                              const std::string& defaultValue)
   {
-    boost::mutex::scoped_lock lock(mutex_);
-
     std::string value;
-    if (db_.LookupGlobalProperty(value, property))
+
+    if (LookupGlobalProperty(value, property))
     {
       return value;
     }
